@@ -12,7 +12,8 @@ class Invalid(Exception):
     the value for a particular node was not valid.
 
     The constructor receives a mandatory ``node`` argument.  This must
-    be an instance of the :class:`colander.SchemaNode` class.
+    be an instance of the :class:`colander.SchemaNode` class, or at
+    least something with the same interface.
 
     The constructor also receives an optional ``msg`` keyword
     argument, defaulting to ``None``.  The ``msg`` argument is a
@@ -27,12 +28,53 @@ class Invalid(Exception):
         self.msg = msg
         self.children = []
 
-    def add(self, exc):
+    def add(self, exc, pos=None):
         """ Add a child exception; ``exc`` must be an instance of
-        :class:`colander.Invalid`"""
+        :class:`colander.Invalid` or a subclass.
+
+        ``pos`` is a value important for accurate error reporting.  If
+        it is provided, it must be an integer representing the
+        position of ``exc`` relative to all other subexceptions of
+        this exception node.  For example, if the exception being
+        added is about the third child of the exception which is
+        ``self``, ``pos`` might be passed as ``3``.
+
+        If ``pos`` is provided, it will be assigned to the ``pos``
+        attribute of the provided ``exc`` object.
+
+        The ``parent`` attribute of the provided ``exc`` will be set
+        as a reference to ``self``.
+        """
         exc.parent = self
+        if pos is not None:
+            exc.pos = pos
         self.children.append(exc)
 
+    def __setitem__(self, name, msg):
+        """ Add a subexception related to a child node with the
+        message ``msg``. ``name`` must be present in the names of the
+        set of child nodes of this exception's node; if this is not
+        so, a :exc:`KeyError` is raised.
+
+        For example, if the exception upon which ``__setitem__`` is
+        called has a node attribute with children, and that node
+        attribute has children that have the names ``name`` and
+        ``title``, you may successfully call ``__setitem__('name',
+        'Bad name')`` or ``__setitem__('title', 'Bad title')``.  But
+        calling ``__setitem__('wrong', 'whoops')`` will result in a
+        :exc:`KeyError`.
+
+        This method is typically only useful if the ``node`` attribute
+        of the exception upon which it is called is a schema node
+        representing a mapping.
+        """
+        for num, child in enumerate(self.node.children):
+            if child.name == name:
+                exc = Invalid(child, msg)
+                self.add(exc, num)
+                return
+        raise KeyError(name)
+
     def paths(self):
         """ Return all paths through the exception graph  """
         def traverse(node, stack):
@@ -147,43 +189,95 @@ class OneOf(object):
             raise Invalid(node, '"%s" is not one of %s' % (
                 value, ', '.join(['%s' % x for x in self.values])))
 
-class Mapping(object):
+class Type(object):
+    """ Abstract base class for types (only for convenience) """
+    def pserialize(self, node, value):
+        return self.serialize(node, value)
+
+    def pdeserialize(self, node, value):
+        return self.deserialize(node, value)
+
+class Mapping(Type):
     """ A type which represents a mapping of names to nodes.
 
     The subnodes of the :class:`colander.SchemaNode` that wraps
     this type imply the named keys and values in the mapping.
 
-    The constructor of a mapping type accepts a single optional
-    keyword argument named ``unknown_keys``.  By default, this
-    argument is ``ignore``.
+    The constructor of this type accepts two extra optional keyword
+    arguments that other types do not: ``unknown`` and ``partial``.
+    Attributes of the same name can be set on a type instance to
+    control the behavior after construction.
 
-    The potential values of ``unknown_keys`` are:
+    unknown
+        ``unknown`` controls the behavior of this type when an unknown
+        key is encountered in the value passed to the ``serialize`` or
+        ``deserialize`` methods of this instance.  The potential
+        values of ``unknown`` are:
 
-    - ``ignore`` means that keys that are not present in the schema
-      associated with this type will be ignored during
-      deserialization.
+        - ``ignore`` means that keys that are not present in the schema
+          associated with this type will be ignored during
+          deserialization.
 
-    - ``raise`` will cause a :exc:`colander.Invalid` exception to be
-      raised when unknown keys are present during deserialization.
+        - ``raise`` will cause a :exc:`colander.Invalid` exception to be
+          raised when unknown keys are present during deserialization.
 
-    - ``preserve`` will preserve the 'raw' unknown keys and values in
-      the returned data structure during deserialization.
+        - ``preserve`` will preserve the 'raw' unknown keys and values
+          in the returned data structure.
+
+        Default: ``ignore``.
+
+        Note that the ``pserialize`` and ``pdeserialize`` methods of
+        this type will override this behavior, behaving as if
+        ``unknown`` is ``ignore`` for the duration of those method calls.
+
+    partial
+        ``partial`` controls the behavior of this type when a
+        schema-expected key is missing from the value passed to the
+        ``serialize`` and ``deserialize`` methods of this instance.
+        During serialization and deserialization, when ``partial`` is
+        ``False``, a :exc:`colander.Invalid` exception will be raised
+        if the mapping value does not contain a key specified by the
+        schema node related to this mapping type.  When ``partial`` is
+        ``True``, no exception is raised and a partial mapping will
+        be serialized/deserialized.
+
+        Default: ``False``.
+
+        Note that the ``pserialize`` and ``pdeserialize`` methods of
+        this type will override this behavior, behaving as if
+        ``partial`` is ``True`` for the duration of those method calls.
     """
 
-    def __init__(self, unknown_keys='ignore'):
-        if not unknown_keys in ['ignore', 'raise', 'preserve']:
+    def __init__(self, unknown='ignore', partial=False):
+        self.unknown = unknown
+        self.partial = partial
+
+    def _set_unknown(self, value):
+        if not value in ['ignore', 'raise', 'preserve']:
             raise ValueError(
-                'unknown_keys argument must be one of "ignore", "raise", '
+                'unknown attribute must be one of "ignore", "raise", '
                 'or "preserve"')
-        self.unknown_keys = unknown_keys
-        
+        self._unknown = value
+
+    def _get_unknown(self):
+        return self._unknown
+
+    unknown = property(_get_unknown, _set_unknown)
+
     def _validate(self, node, value):
         try:
             return dict(value)
         except Exception, e:
             raise Invalid(node, '%r is not a mapping type: %s' % (value, e))
 
-    def _impl(self, node, value, callback, default_callback):
+    def _impl(self, node, value, callback, default_callback, unknown=None,
+              partial=None):
+        if partial is None:
+            partial = self.partial
+
+        if unknown is None:
+            unknown = self.unknown
+
         value = self._validate(node, value)
 
         error = None
@@ -196,24 +290,26 @@ class Mapping(object):
             try:
                 if subval is _missing:
                     if subnode.required:
-                        raise Invalid(
-                            subnode,
-                            '"%s" is required but missing' % subnode.name)
+                        if not partial:
+                            raise Invalid(
+                                subnode,
+                                '"%s" is required but missing' % subnode.name)
+                        else:
+                            continue
                     result[name] = default_callback(subnode)
                 else:
                     result[name] = callback(subnode, subval)
             except Invalid, e:
                 if error is None:
                     error = Invalid(node)
-                e.pos = num
-                error.add(e)
+                error.add(e, num)
 
-        if self.unknown_keys == 'raise':
+        if unknown == 'raise':
             if value:
                 raise Invalid(node,
                               'Unrecognized keys in mapping: %r' % value)
 
-        elif self.unknown_keys == 'preserve':
+        elif unknown == 'preserve':
             result.update(value)
 
         if error is not None:
@@ -221,6 +317,13 @@ class Mapping(object):
                 
         return result
 
+    def serialize(self, node, value):
+        def callback(subnode, subval):
+            return subnode.serialize(subval)
+        def default_callback(subnode):
+            return subnode.serialize(subnode.default)
+        return self._impl(node, value, callback, default_callback)
+
     def deserialize(self, node, value):
         def callback(subnode, subval):
             return subnode.deserialize(subval)
@@ -228,12 +331,23 @@ class Mapping(object):
             return subnode.default
         return self._impl(node, value, callback, default_callback)
 
-    def serialize(self, node, value):
+    def pserialize(self, node, value):
         def callback(subnode, subval):
-            return subnode.serialize(subval)
+            return subnode.pserialize(subval)
         def default_callback(subnode):
             return subnode.serialize(subnode.default)
-        return self._impl(node, value, callback, default_callback)
+        return self._impl(
+            node, value, callback, default_callback, unknown='ignore',
+            partial=True)
+
+    def pdeserialize(self, node, value):
+        def callback(subnode, subval):
+            return subnode.pdeserialize(subval)
+        def default_callback(subnode):
+            return subnode.default
+        return self._impl(
+            node, value, callback, default_callback, unknown='ignore',
+            partial=True)
 
 class Positional(object):
     """
@@ -243,7 +357,7 @@ class Positional(object):
     creating a dictionary representation of an error tree.
     """
 
-class Tuple(Positional):
+class Tuple(Type, Positional):
     """ A type which represents a fixed-length sequence of nodes.
 
     The subnodes of the :class:`colander.SchemaNode` that wraps
@@ -280,8 +394,7 @@ class Tuple(Positional):
             except Invalid, e:
                 if error is None:
                     error = Invalid(node)
-                e.pos = num
-                error.add(e)
+                error.add(e, num)
                 
         if error is not None:
             raise error
@@ -298,7 +411,7 @@ class Tuple(Positional):
             return subnode.serialize(subval)
         return self._impl(node, value, callback)
 
-class Sequence(Positional):
+class Sequence(Type, Positional):
     """
     A type which represents a variable-length sequence of nodes,
     all of which must be of the same type.
@@ -308,28 +421,36 @@ class Sequence(Positional):
     sequence type.
 
     The optional ``accept_scalar`` argument to this type's constructor
-    indicates that if the value found during serialization or
+    indicates what should happen if the value found during serialization or
     deserialization does not have an ``__iter__`` method or is a
-    mapping type, that the value will be converted to a length-one
-    list.  If ``accept_scalar`` is ``False`` (the default), and the
-    value does not have an ``__iter__`` method, an
+    mapping type.
+
+    If ``accept_scalar`` is ``True`` and the value does not have an
+    ``__iter__`` method or is a mapping type, the value will be turned
+    into a single element list.
+
+    If ``accept_scalar`` is ``False`` and the value does not have an
+    ``__iter__`` method or is a mapping type, an
     :exc:`colander.Invalid` error will be raised during serialization
     and deserialization.
 
+    The default value of ``accept_scalar`` is ``False``.
     """
     def __init__(self, accept_scalar=False):
         self.accept_scalar = accept_scalar
 
-    def _validate(self, node, value):
+    def _validate(self, node, value, accept_scalar):
         if hasattr(value, '__iter__') and not hasattr(value, 'get'):
             return list(value)
-        if self.accept_scalar:
+        if accept_scalar:
             return [value]
         else:
             raise Invalid(node, '%r is not iterable' % value)
 
-    def _impl(self, node, value, callback):
-        value = self._validate(node, value)
+    def _impl(self, node, value, callback, accept_scalar):
+        if accept_scalar is None:
+            accept_scalar = self.accept_scalar
+        value = self._validate(node, value, accept_scalar)
 
         error = None
         result = []
@@ -339,29 +460,66 @@ class Sequence(Positional):
             except Invalid, e:
                 if error is None:
                     error = Invalid(node)
-                e.pos = num
-                error.add(e)
+                error.add(e, num)
                 
         if error is not None:
             raise error
 
         return result
 
-    def deserialize(self, node, value):
+    def deserialize(self, node, value, accept_scalar=None):
+        """
+        Along with the normal ``node`` and ``value`` arguments, this
+        method accepts an additional optional keyword argument:
+        ``accept_scalar``.  This keyword argument can be used to
+        override the constructor value of the same name.
+
+        If ``accept_scalar`` is ``True`` and the ``value`` does not
+        have an ``__iter__`` method or is a mapping type, the value
+        will be turned into a single element list.
+
+        If ``accept_scalar`` is ``False`` and the value does not have an
+        ``__iter__`` method or is a mapping type, an
+        :exc:`colander.Invalid` error will be raised during serialization
+        and deserialization.
+
+        The default of ``accept_scalar`` is ``None``, which means
+        respect the default ``accept_scalar`` value attached to this
+        instance via its constructor.
+        """
         def callback(subnode, subval):
             return subnode.deserialize(subval)
-        return self._impl(node, value, callback)
+        return self._impl(node, value, callback, accept_scalar)
 
-    def serialize(self, node, value):
+    def serialize(self, node, value, accept_scalar=None):
+        """
+        Along with the normal ``node`` and ``value`` arguments, this
+        method accepts an additional optional keyword argument:
+        ``accept_scalar``.  This keyword argument can be used to
+        override the constructor value of the same name.
+
+        If ``accept_scalar`` is ``True`` and the ``value`` does not
+        have an ``__iter__`` method or is a mapping type, the value
+        will be turned into a single element list.
+
+        If ``accept_scalar`` is ``False`` and the value does not have an
+        ``__iter__`` method or is a mapping type, an
+        :exc:`colander.Invalid` error will be raised during serialization
+        and deserialization.
+
+        The default of ``accept_scalar`` is ``None``, which means
+        respect the default ``accept_scalar`` value attached to this
+        instance via its constructor.
+        """
         def callback(subnode, subval):
             return subnode.serialize(subval)
-        return self._impl(node, value, callback)
+        return self._impl(node, value, callback, accept_scalar)
 
 Seq = Sequence
 
 default_encoding = 'utf-8'
 
-class String(object):
+class String(Type):
     """ A type representing a Unicode string.  This type constructor
     accepts a single argument ``encoding``, representing the encoding
     which should be applied to object serialization.  It defaults to
@@ -394,7 +552,7 @@ class String(object):
 
 Str = String
 
-class Integer(object):
+class Integer(Type):
     """ A type representing an integer.
 
     The subnodes of the :class:`colander.SchemaNode` that wraps
@@ -418,7 +576,7 @@ class Integer(object):
 
 Int = Integer
 
-class Float(object):
+class Float(Type):
     """ A type representing a float.
 
     The subnodes of the :class:`colander.SchemaNode` that wraps
@@ -442,7 +600,7 @@ class Float(object):
 
 Int = Integer
 
-class Boolean(object):
+class Boolean(Type):
     """ A type representing a boolean object.
 
     During deserialization, a value in the set (``false``, ``0``) will
@@ -475,7 +633,7 @@ class Boolean(object):
 
 Bool = Boolean
 
-class GlobalObject(object):
+class GlobalObject(Type):
     """ A type representing an importable Python object.  This type
     serializes 'global' Python objects (objects which can be imported)
     to dotted Python names.
@@ -582,7 +740,7 @@ class GlobalObject(object):
         except AttributeError:
             raise Invalid(node, '%r has no __name__' % value)
 
-class DateTime(object):
+class DateTime(Type):
     """ A type representing a Python ``datetime.datetime`` object.
 
     This type serializes python ``datetime.datetime`` objects to a
@@ -590,13 +748,22 @@ class DateTime(object):
     The format includes the date, the time, and the timezone of the
     datetime.
 
-    The constructor accepts a single argument named ``default_tzinfo``
-    which should be a Python ``tzinfo`` object; by default it is None,
+    The constructor accepts an argument named ``default_tzinfo`` which
+    should be a Python ``tzinfo`` object; by default it is None,
     meaning that the default tzinfo will be equivalent to UTC (Zulu
     time).  The ``default_tzinfo`` tzinfo object is used to convert
     'naive' datetimes to a timezone-aware representation during
     serialization.
 
+    You can adjust the error message reported by this class by
+    changing its ``err_template`` attribute in a subclass on an
+    instance of this class.  By default, the ``err_tempalte``
+    attribute is the string ``%(value)s cannot be parsed as an iso8601
+    date: %(exc)s``.  This string is used as the interpolation subject
+    of a dictionary composed of ``value`` and ``exc``.  ``value`` and
+    ``exc`` are the unvalidatable value and the exception caused
+    trying to convert the value, respectively.
+
     For convenience, this type is also willing to coerce
     ``datetime.date`` objects to a DateTime ISO string representation
     during serialization.  It does so by using midnight of the day as
@@ -611,6 +778,7 @@ class DateTime(object):
     The subnodes of the :class:`colander.SchemaNode` that wraps
     this type are ignored.
     """
+    err_template =  '%(value)s cannot be parsed as an iso8601 date: %(exc)s'
     def __init__(self, default_tzinfo=None):
         if default_tzinfo is None:
             default_tzinfo = iso8601.iso8601.Utc()
@@ -634,12 +802,11 @@ class DateTime(object):
                 result = datetime.datetime(year, month, day,
                                            tzinfo=self.default_tzinfo)
             except Exception, e:
-                raise Invalid(node,
-                              '%s cannot be parsed as an iso8601 date: %s' %
-                              (value, e))
+                raise Invalid(node, self.err_template % {'value':value,
+                                                         'exc':e})
         return result
 
-class Date(object):
+class Date(Type):
     """ A type representing a Python ``datetime.date`` object.
 
     This type serializes python ``datetime.date`` objects to a
@@ -648,6 +815,15 @@ class Date(object):
 
     The constructor accepts no arguments.
 
+    You can adjust the error message reported by this class by
+    changing its ``err_template`` attribute in a subclass on an
+    instance of this class.  By default, the ``err_template``
+    attribute is the string ``%(value)s cannot be parsed as an iso8601
+    date: %(exc)s``.  This string is used as the interpolation subject
+    of a dictionary composed of ``value`` and ``exc``.  ``value`` and
+    ``exc`` are the unvalidatable value and the exception caused
+    trying to convert the value, respectively.
+
     For convenience, this type is also willing to coerce
     ``datetime.datetime`` objects to a date-only ISO string
     representation during serialization.  It does so by stripping off
@@ -663,6 +839,7 @@ class Date(object):
     The subnodes of the :class:`colander.SchemaNode` that wraps
     this type are ignored.
     """
+    err_template =  '%(value)s cannot be parsed as an iso8601 date: %(exc)s'
     def serialize(self, node, value):
         if isinstance(value, datetime.datetime):
             value = value.date()
@@ -679,9 +856,8 @@ class Date(object):
                 year, month, day = map(int, value.split('-', 2))
                 result = datetime.date(year, month, day)
             except Exception, e:
-                raise Invalid(node,
-                              '%s cannot be parsed as an iso8601 date: %s' %
-                              (value, e))
+                raise Invalid(node, self.err_template % {'value':value,
+                                                         'exc':e})
         return result
 
 class SchemaNode(object):
@@ -754,20 +930,30 @@ class SchemaNode(object):
         return self.typ.serialize(self, self.default)
 
     def deserialize(self, value):
-        """ Derialize the value based on the schema represented by this
-        node """
+        """ Deserialize the value based on the schema represented by
+        this node. """
         value = self.typ.deserialize(self, value)
         if self.validator is not None:
             self.validator(self, value)
         return value
 
     def serialize(self, value):
-        """ Serialize the value based on the schema represented by this
-        node """
+        """ Serialize the value based on the schema represented by
+        this node."""
         return self.typ.serialize(self, value)
 
+    def pserialize(self, value):
+        """ Partially serialize the value based on the schema
+        represented by this node.  """
+        return self.typ.pserialize(self, value)
+
+    def pdeserialize(self, value):
+        """ Partially deserialize the value based on the schema
+        represented by this node.  """
+        return self.typ.pdeserialize(self, value)
+
     def add(self, node):
-        """ Add a subnode to this node """
+        """ Add a subnode to this node. """
         self.children.append(node)
 
     def __getitem__(self, name):
@@ -813,8 +999,8 @@ class Schema(object):
         node = object.__new__(cls.node_type)
         node.name = None
         node._order = SchemaNode._counter.next()
-        typ = cls.schema_type(*args, **kw)
-        node.__init__(typ)
+        typ = cls.schema_type()
+        node.__init__(typ, *args, **kw)
         for n in cls.nodes:
             node.add(n)
         return node
