@@ -99,7 +99,24 @@ class TestInvalid(unittest.TestCase):
             result,
             "{'node1.node2.3': 'exc1; exc2; exc3', 'node1.node4': 'exc1; exc4'}"
             )
-        
+
+    def test___setitem__fails(self):
+        node = DummySchemaNode(None)
+        exc = self._makeOne(node, 'msg')
+        self.assertRaises(KeyError, exc.__setitem__, 'notfound', 'msg')
+
+    def test___setitem__succeeds(self):
+        node = DummySchemaNode(None)
+        child = DummySchemaNode(None)
+        child.name = 'found'
+        node.children = [child]
+        exc = self._makeOne(node, 'msg')
+        exc['found'] = 'msg2'
+        self.assertEqual(len(exc.children), 1)
+        childexc = exc.children[0]
+        self.assertEqual(childexc.pos, 0)
+        self.assertEqual(childexc.node.name, 'found')
+
 class TestAll(unittest.TestCase):
     def _makeOne(self, validators):
         from colander import All
@@ -194,15 +211,32 @@ class TestOneOf(unittest.TestCase):
         e = invalid_exc(validator, None, None)
         self.assertEqual(e.msg, '"None" is not one of 1, 2')
 
-class TestMapping(unittest.TestCase):
-    def _makeOne(self, unknown_keys='ignore'):
-        from colander import Mapping
-        return Mapping(unknown_keys=unknown_keys)
+class TestType(unittest.TestCase):
+    def _makeOne(self):
+        from colander import Type
+        return Type()
 
-    def test_ctor_bad_unknown_keys(self):
+    def test_pserialize(self):
+        typ = self._makeOne()
+        typ.serialize = lambda *arg: arg
+        result = typ.pserialize(None, None)
+        self.assertEqual(result, (None, None))
+
+    def test_pdeserialize(self):
+        typ = self._makeOne()
+        typ.deserialize = lambda *arg: arg
+        result = typ.pdeserialize(None, None)
+        self.assertEqual(result, (None, None))
+
+class TestMapping(unittest.TestCase):
+    def _makeOne(self, *arg, **kw):
+        from colander import Mapping
+        return Mapping(*arg, **kw)
+
+    def test_ctor_bad_unknown(self):
         self.assertRaises(ValueError, self._makeOne, 'badarg')
 
-    def test_ctor_good_unknown_keys(self):
+    def test_ctor_good_unknown(self):
         try:
             self._makeOne('ignore')
             self._makeOne('raise')
@@ -230,17 +264,17 @@ class TestMapping(unittest.TestCase):
         result = typ.deserialize(node, {'a':1})
         self.assertEqual(result, {'a':1})
 
-    def test_deserialize_unknown_keys_raise(self):
+    def test_deserialize_unknown_raise(self):
         node = DummySchemaNode(None)
         node.children = [DummySchemaNode(None, name='a')]
-        typ = self._makeOne('raise')
+        typ = self._makeOne(unknown='raise')
         e = invalid_exc(typ.deserialize, node, {'a':1, 'b':2})
         self.assertEqual(e.msg, "Unrecognized keys in mapping: {'b': 2}")
 
-    def test_deserialize_unknown_keys_preserve(self):
+    def test_deserialize_unknown_preserve(self):
         node = DummySchemaNode(None)
         node.children = [DummySchemaNode(None, name='a')]
-        typ = self._makeOne('preserve')
+        typ = self._makeOne(unknown='preserve')
         result = typ.deserialize(node, {'a':1, 'b':2})
         self.assertEqual(result, {'a':1, 'b':2})
 
@@ -275,6 +309,16 @@ class TestMapping(unittest.TestCase):
         e = invalid_exc(typ.deserialize, node, {'a':1})
         self.assertEqual(e.children[0].msg, '"b" is required but missing')
 
+    def test_deserialize_subnode_partial(self):
+        node = DummySchemaNode(None)
+        node.children = [
+            DummySchemaNode(None, name='a'),
+            DummySchemaNode(None, name='b'),
+            ]
+        typ = self._makeOne(partial=True)
+        result = typ.deserialize(node, {'a':1})
+        self.assertEqual(result, {'a':1})
+
     def test_serialize_not_a_mapping(self):
         node = DummySchemaNode(None)
         typ = self._makeOne()
@@ -295,17 +339,17 @@ class TestMapping(unittest.TestCase):
         result = typ.serialize(node, {'a':1})
         self.assertEqual(result, {'a':1})
 
-    def test_serialize_unknown_keys_raise(self):
+    def test_serialize_unknown_raise(self):
         node = DummySchemaNode(None)
         node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne('raise')
         e = invalid_exc(typ.serialize, node, {'a':1, 'b':2})
         self.assertEqual(e.msg, "Unrecognized keys in mapping: {'b': 2}")
 
-    def test_serialize_unknown_keys_preserve(self):
+    def test_serialize_unknown_preserve(self):
         node = DummySchemaNode(None)
         node.children = [DummySchemaNode(None, name='a')]
-        typ = self._makeOne('preserve')
+        typ = self._makeOne(unknown='preserve')
         result = typ.serialize(node, {'a':1, 'b':2})
         self.assertEqual(result, {'a':1, 'b':2})
 
@@ -340,6 +384,56 @@ class TestMapping(unittest.TestCase):
         e = invalid_exc(typ.serialize, node, {'a':1})
         self.assertEqual(e.children[0].msg, '"b" is required but missing')
 
+    def test_serialize_subnode_partial(self):
+        node = DummySchemaNode(None)
+        node.children = [
+            DummySchemaNode(None, name='a'),
+            DummySchemaNode(None, name='b'),
+            ]
+        typ = self._makeOne(partial=True)
+        result = typ.serialize(node, {'a':1})
+        self.assertEqual(result, {'a':1})
+
+    def test_pserialize(self):
+        node = DummySchemaNode(None)
+        node.children = [
+            DummySchemaNode(None, name='a'),
+            DummySchemaNode(None, name='b'),
+            ]
+        typ = self._makeOne()
+        result = typ.pserialize(node, {'a':1})
+        self.assertEqual(result, {'a':1})
+
+    def test_pserialize_using_default(self):
+        node = DummySchemaNode(None)
+        node.children = [
+            DummySchemaNode(None, name='a'),
+            DummySchemaNode(None, name='b', default='abc'),
+            ]
+        typ = self._makeOne()
+        result = typ.pserialize(node, {'a':1})
+        self.assertEqual(result, {'a':1, 'b':'abc'})
+
+    def test_pdeserialize(self):
+        node = DummySchemaNode(None)
+        node.children = [
+            DummySchemaNode(None, name='a'),
+            DummySchemaNode(None, name='b'),
+            ]
+        typ = self._makeOne()
+        result = typ.pdeserialize(node, {'a':1})
+        self.assertEqual(result, {'a':1})
+
+    def test_pdeserialize_using_default(self):
+        node = DummySchemaNode(None)
+        node.children = [
+            DummySchemaNode(None, name='a'),
+            DummySchemaNode(None, name='b', default='abc'),
+            ]
+        typ = self._makeOne()
+        result = typ.pdeserialize(node, {'a':1})
+        self.assertEqual(result, {'a':1, 'b':'abc'})
+
 class TestTuple(unittest.TestCase):
     def _makeOne(self):
         from colander import Tuple
@@ -1105,12 +1199,24 @@ class TestSchemaNode(unittest.TestCase):
         e = invalid_exc(node.deserialize, 1)
         self.assertEqual(e.msg, 'Wrong')
 
+    def test_pdeserialize(self):
+        typ = DummyType()
+        node = self._makeOne(typ)
+        result = node.pdeserialize(1)
+        self.assertEqual(result, 1)
+
     def test_serialize(self):
         typ = DummyType()
         node = self._makeOne(typ)
         result = node.serialize(1)
         self.assertEqual(result, 1)
 
+    def test_pserialize(self):
+        typ = DummyType()
+        node = self._makeOne(typ)
+        result = node.pserialize(1)
+        self.assertEqual(result, 1)
+
     def test_add(self):
         node = self._makeOne(None)
         node.add(1)
@@ -1163,11 +1269,11 @@ class TestSchema(unittest.TestCase):
         class MySchema(colander.Schema):
             thing = colander.SchemaNode(colander.String())
             thing2 = colander.SchemaNode(colander.String(), title='bar')
-        node = MySchema(unknown_keys='raise')
+        node = MySchema(default='abc')
         self.failUnless(hasattr(node, '_order'))
+        self.assertEqual(node.default, 'abc')
         self.assertEqual(node.__class__, colander.SchemaNode)
         self.assertEqual(node.typ.__class__, colander.Mapping)
-        self.assertEqual(node.typ.unknown_keys, 'raise')
         self.assertEqual(node.children[0].typ.__class__, colander.String) 
         self.assertEqual(node.children[0].title, 'Thing')
         self.assertEqual(node.children[1].title, 'bar')
@@ -1356,7 +1462,10 @@ class DummySchemaNode(object):
         if self.exc:
             raise Invalid(self, self.exc)
         return val
-        
+
+    pserialize = serialize
+    pdeserialize = deserialize
+
 class DummyValidator(object):
     def __init__(self, msg=None):
         self.msg = msg
@@ -1378,4 +1487,10 @@ class DummyType(object):
 
     def deserialize(self, node, value):
         return value
+
+    def pserialize(self, node, value):
+        return value
     
+    def pdeserialize(self, node, value):
+        return value
+
